@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CmdExec.cc,v 1.108 2004/05/11 07:31:48 lav Exp $ */
+/* $Id: CmdExec.cc,v 1.115 2005/08/12 07:05:32 lav Exp $ */
 
 #include <config.h>
 #include <errno.h>
@@ -60,7 +60,8 @@ static ResDecl
    res_interactive	   ("cmd:interactive",	"no", ResMgr::BoolValidate,ResMgr::NoClosure),
    res_move_background	   ("cmd:move-background","yes", ResMgr::BoolValidate,ResMgr::NoClosure),
    res_set_term_status     ("cmd:set-term-status","no", ResMgr::BoolValidate,0),
-   res_term_status         ("cmd:term-status",  "", 0, 0);
+   res_term_status         ("cmd:term-status",  "", 0, 0),
+   res_trace		   ("cmd:trace",  "no",	ResMgr::BoolValidate,ResMgr::NoClosure);
 
 CmdExec	 *CmdExec::cwd_owner=0;
 CmdExec	 *CmdExec::chain=0;
@@ -94,7 +95,7 @@ int  CmdExec::RestoreCWD()
 void CmdExec::FeedCmd(const char *c)
 {
    partial_cmd=false;
-   time(&start_time);
+   start_time=now;
    if(cmd_buf==0)
    {
       cmd_buf=next_cmd=xstrdup(c);
@@ -108,7 +109,7 @@ void CmdExec::FeedCmd(const char *c)
 
 void CmdExec::PrependCmd(const char *c)
 {
-   time(&start_time);
+   start_time=now;
 
    int len=strlen(c);
    int nl=(len>0 && c[len-1]!='\n');
@@ -188,6 +189,13 @@ void  CmdExec::exec_parsed_command()
 
    bool did_default=false;
 
+   if(ResMgr::QueryBool("cmd:trace",0))
+   {
+      char *c=args->CombineQuoted();
+      printf("+ %s\n",c);
+      xfree(c);
+   }
+
 restart:
 
    const struct cmd_rec *c;
@@ -310,7 +318,7 @@ void CmdExec::RemoveFeeder()
 
    // save old cwd if necessary
    if(interactive && feeder->prev==0)
-      cwd_history.Set(session,session->GetCwd());
+      cwd_history.Set(session);
 
    CmdFeeder *tmp=feeder->prev;
    next_cmd=cmd_buf=feeder->saved_buf;
@@ -386,7 +394,7 @@ int CmdExec::Do()
 	       // cd to another url.
 	       const char *loc_c=session->GetNewLocation();
 	       int max_redirections=ResMgr::Query("xfer:max-redirections",0);
-	       if(loc_c && max_redirections>0 && last_char(loc_c)=='/')
+	       if(loc_c && max_redirections>0)
 	       {
 		  eprintf(_("%s: received redirection to `%s'\n"),"cd",loc_c);
 		  if(++redirections>max_redirections)
@@ -396,17 +404,18 @@ int CmdExec::Do()
 		  }
 
 		  char *loc=alloca_strdup(loc_c);
-		  session->Close(); // loc_c is no longer valid.
-
 		  ParsedURL u(loc,true);
-
 		  if(!u.proto)
 		  {
-		     url::decode_string(loc);
-		     session->Chdir(loc);
+		     bool is_file=(last_char(loc)!='/');
+		     FileAccess::Path new_cwd(session->GetNewCwd());
+		     new_cwd.Change(0,is_file,loc);
+		     session->PathVerify(new_cwd);
 		     Roll(session);
 		     return MOVED;
 		  }
+		  session->Close();
+		  exit_code=0;
 		  builtin=BUILTIN_NONE;
 		  char *cmd=string_alloca(6+3+2*strlen(loc));
 		  strcpy(cmd,"open \"");
@@ -515,12 +524,7 @@ int CmdExec::Do()
 	       if(status_line)
 		  status_line->Clear();
 	       if(builtin==BUILTIN_CD)
-	       {
-		  // accept the path
-		  const char *f=session->GetFile();
-		  f=alloca_strdup(f);
-		  session->Chdir(f,false);
-	       }
+		  session->ChdirAccept();
 	       session->Close();
 	       exit_code=0;
 	       builtin=BUILTIN_NONE;
@@ -616,6 +620,8 @@ try_get_cmd:
 
 	 char *prompt=MakePrompt();
 	 feeder_called=true;
+	 if(fg)
+	    feeder->Fg();
 	 const char *cmd=feeder->NextCmd(this,prompt);
 	 feeder_called=false;
 	 if(cmd==0)
@@ -673,6 +679,8 @@ try_get_cmd:
       partial_cmd=true;
       goto try_get_cmd;
    case(PARSE_OK):
+      if(feeder)
+	 feeder->Bg();
       break;
    }
    if(args==0 || args->count()==0)
@@ -724,7 +732,7 @@ void CmdExec::PrintStatus(int v,const char *prefix)
    }
    if(queue_feeder)
    {
-      if(suspended)
+      if(IsSuspended())
 	 printf("%s%s\n",prefix,_("Queue is stopped."));
       for(int i=0; i<waiting_num; i++)
       {
@@ -802,7 +810,6 @@ CmdExec::CmdExec(FileAccess *f,LocalDirectory *c) : SessionJob(f)
    verify_path_cached=false;
 
    start_time=0;
-   old_cwd=0;
    old_lcwd=0;
    slot=0;
 
@@ -840,7 +847,6 @@ CmdExec::~CmdExec()
    delete cwd;
    if(cwd_owner==this)
       cwd_owner=0;
-   xfree(old_cwd);
    xfree(old_lcwd);
    delete glob;
    delete args_glob;
@@ -923,7 +929,7 @@ char *CmdExec::MakePrompt()
 void CmdExec::beep_if_long()
 {
    if(start_time!=0 && long_running!=0
-   && time(0)-start_time>long_running
+   && now-start_time>long_running
    && interactive && Idle() && isatty(1))
       write(1,"\007",1);
 }
