@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileAccess.cc,v 1.120 2005/04/04 09:59:38 lav Exp $ */
+/* $Id: FileAccess.cc,v 1.124 2005/08/04 13:10:40 lav Exp $ */
 
 #include <config.h>
 
@@ -49,7 +49,6 @@ FileAccess *FileAccess::chain=0;
 void FileAccess::Init()
 {
    vproto=0;
-   home=0;
    portname=0;
    hostname=0;
    user=pass=0;
@@ -60,7 +59,8 @@ void FileAccess::Init()
    file1=0;
    real_cwd=0;
    default_cwd="~";
-   cwd=xstrdup(default_cwd);
+   cwd.Set(default_cwd,false,0);
+   new_cwd=0;
    real_pos=-1;
    pos=0;
    mode=CLOSED;
@@ -101,10 +101,8 @@ FileAccess::FileAccess(const FileAccess *fa)
 {
    Init();
 
-   xfree(cwd);
-   cwd=xstrdup(fa->cwd);
-   xfree(home);
-   home=xstrdup(fa->home);
+   cwd=fa->cwd;
+   home=fa->home;
    xfree(user);
    user=xstrdup(fa->user);
    xfree(pass);
@@ -122,10 +120,8 @@ FileAccess::~FileAccess()
    xfree(vproto);
    xfree(file);
    xfree(file_url);
-   xfree(cwd);
    xfree(real_cwd);
    xfree(error);
-   xfree(home);
    xfree(user);
    xfree(pass);
    xfree(hostname);
@@ -339,6 +335,7 @@ void FileAccess::Close()
    xfree(file); file=0;
    xfree(file_url); file_url=0;
    xfree(file1); file1=0;
+   delete new_cwd; new_cwd=0;
    mode=CLOSED;
    opt_date=0;
    opt_size=0;
@@ -401,8 +398,21 @@ const char *FileAccess::GetFileURL(const char *f,int flags)
    u.port=portname;
    if(!(flags&NO_PATH))
    {
+      if(cwd.url)
+      {
+	 Path f_path(cwd);
+	 if(f)
+	    f_path.Change(f,true);
+	 int f_path_index=url::path_index(f_path.url);
+	 xfree(url);
+	 url=u.Combine(home);
+	 url=(char*)xrealloc(url,strlen(url)+strlen(f_path.url+f_path_index)+2);
+	 strcpy(url+strlen(url),f_path.url+f_path_index);
+	 return url;
+      }
+
       if(!f || (f[0]!='/' && f[0]!='~'))
-	 f=dir_file(cwd?cwd:"~",f);
+	 f=dir_file(cwd.path?cwd.path:"~",f);
       u.path=(char*)f;
    }
 
@@ -470,8 +480,8 @@ void FileAccess::AnonymousLogin()
 
 void FileAccess::ResetLocationData()
 {
-   xfree(cwd);  cwd=xstrdup(default_cwd);
-   xfree(home); home=0;
+   cwd.Set(default_cwd,false,0);
+   home.Set((char*)0);
 }
 
 void FileAccess::SetPasswordGlobal(const char *p)
@@ -523,16 +533,15 @@ void  FileAccess::ExpandTildeInCWD()
 {
    if(home)
    {
-      if(cwd)
-	 expand_tilde(&cwd,home);
+      cwd.ExpandTilde(home);
+      if(new_cwd)
+	 new_cwd->ExpandTilde(home);
       if(real_cwd)
 	 expand_tilde(&real_cwd,home);
       if(file)
 	 expand_tilde(&file,home);
       if(file1)
 	 expand_tilde(&file1,home);
-      if(mode==CHANGE_DIR)
-	 OptimizePath(file);
    }
 }
 
@@ -572,7 +581,7 @@ int FileAccess::device_prefix_len(const char *path)
    return 0;
 }
 
-void FileAccess::OptimizePath(char *path)
+void FileAccess::Path::Optimize(char *path,int device_prefix_len)
 {
    int	 prefix_size=0;
 
@@ -591,7 +600,7 @@ void FileAccess::OptimizePath(char *path)
    else
    {
       // handle VMS and DOS devices.
-      prefix_size=device_prefix_len(path);
+      prefix_size=device_prefix_len;
    }
 
    char	 *in;
@@ -663,36 +672,27 @@ void FileAccess::OptimizePath(char *path)
 
 void FileAccess::Chdir(const char *path,bool verify)
 {
-   ExpandTildeInCWD();
+   cwd.ExpandTilde(home);
 
-   char	 *newcwd=(char*)alloca(xstrlen(cwd)+strlen(path)+2);
-
-   if(path[0]=='/')
-      strcpy(newcwd,path);
-   else if(path[0]=='~')
-      strcpy(newcwd,path);
-   else if(device_prefix_len(path)>0)
-      strcpy(newcwd,path);
-   else
-   {
-      if(cwd && cwd[0])
-      {
-	 if(last_char(cwd)=='/')
-	    sprintf(newcwd,"%s%s",cwd,path);
-	 else
-	    sprintf(newcwd,"%s/%s",cwd,path);
-      }
-      else
-	 strcpy(newcwd,path);
-   }
-
-   OptimizePath(newcwd);
-   strip_trailing_slashes(newcwd);
+   delete new_cwd;
+   new_cwd=new Path(&cwd);
+   new_cwd->Change(path,false);
 
    if(verify)
-      Open(newcwd,CHANGE_DIR);
+      Open(new_cwd->path,CHANGE_DIR);
    else
-      SetCwd(newcwd);
+   {
+      cwd.Set(new_cwd);
+      delete new_cwd;
+      new_cwd=0;
+   }
+}
+
+void FileAccess::PathVerify(const Path &p)
+{
+   delete new_cwd;
+   new_cwd=new Path(p);
+   Open(new_cwd->path,CHANGE_DIR);
 }
 
 void FileAccess::Chmod(const char *file,int m)
@@ -749,6 +749,14 @@ void FileAccess::SetSuggestedFileName(const char *fn)
    suggested_filename=xstrdup(fn);
 }
 
+void FileAccess::SetFileURL(const char *u)
+{
+   xfree(file_url);
+   file_url=xstrdup(u);
+   if(new_cwd && mode==CHANGE_DIR)
+      new_cwd->SetURL(u);
+}
+
 FileAccess *SessionPool::pool[pool_size];
 
 void SessionPool::Reuse(FileAccess *f)
@@ -1059,7 +1067,6 @@ void FileAccessOperation::SetErrorCached(const char *e)
 ListInfo::ListInfo(FileAccess *s,const char *p)
 {
    session=s;
-   saved_cwd=0;
    result=0;
 
    exclude=0;
@@ -1071,7 +1078,7 @@ ListInfo::ListInfo(FileAccess *s,const char *p)
 
    if(session && p)
    {
-      saved_cwd=xstrdup(session->GetCwd());
+      saved_cwd=session->GetCwd();
       session->Chdir(p,false);
    }
 }
@@ -1080,10 +1087,9 @@ ListInfo::~ListInfo()
 {
    if(session)
       session->Close();
-   if(session && saved_cwd)
+   if(session && saved_cwd.path)
       session->SetCwd(saved_cwd);
    delete result;
-   xfree(saved_cwd);
 }
 
 void ListInfo::SetExclude(const char *p,PatternSet *x)
@@ -1092,6 +1098,126 @@ void ListInfo::SetExclude(const char *p,PatternSet *x)
    exclude_prefix=p;
 }
 
+// Path implementation
+void FileAccess::Path::init()
+{
+   device_prefix_len=0;
+   path=0;
+   is_file=false;
+   url=0;
+}
+FileAccess::Path::~Path()
+{
+   xfree(path);
+   xfree(url);
+}
+void FileAccess::Path::Set(const char *new_path,bool new_is_file,const char *new_url,int new_device_prefix_len)
+{
+   xfree(path);
+   path=xstrdup(new_path);
+   is_file=new_is_file;
+   xfree(url);
+   url=xstrdup(new_url);
+   device_prefix_len=new_device_prefix_len;
+}
+void FileAccess::Path::Set(const Path *o)
+{
+   Set(o->path,o->is_file,o->url,o->device_prefix_len);
+}
+void FileAccess::Path::Change(const char *new_path,bool new_is_file,const char *new_path_enc,int new_device_prefix_len)
+{
+   if(!new_path && new_path_enc)
+   {
+      char *np=alloca_strdup(new_path_enc);
+      url::decode_string(np);
+      new_path=np;
+   }
+   if(!new_path || !*new_path)
+      return;
+   const char *bn=basename_ptr(new_path);
+   if(!strcmp(bn,".") || !strcmp(bn,".."))
+      new_is_file=false;
+   if(url)
+   {
+      int path_index=url::path_index(url);
+      const char *old_url_path=url+path_index;
+      char *new_url_path=string_alloca(strlen(old_url_path)+xstrlen(new_path)*3+2);
+      strcpy(new_url_path,old_url_path);
+      if(is_file)
+      {
+	 dirname_modify(new_url_path);
+	 if(!new_url_path[0])
+	    strcpy(new_url_path,"/~");
+      }
+      if(last_char(new_url_path)!='/')
+	 strcat(new_url_path,"/");
+      if(new_path[0]!='/' && new_path[0]!='~' && new_device_prefix_len==0)
+      {
+	 if(new_path_enc)
+	    strcat(new_url_path,new_path_enc);
+	 else
+	    url::encode_string(new_path,new_url_path+strlen(new_url_path),URL_PATH_UNSAFE);
+      }
+      else
+      {
+	 if(new_path_enc)
+	    strcpy(new_url_path,new_path_enc);
+	 else
+	    url::encode_string(new_path,new_url_path,URL_PATH_UNSAFE);
+      }
+      Optimize(new_url_path+(!strncmp(new_url_path,"/~",2)));
+      url=(char*)xrealloc(url,path_index+strlen(new_url_path)+1);
+      strcpy(url+path_index,new_url_path);
+   }
+   if(new_path[0]!='/' && new_path[0]!='~' && new_device_prefix_len==0
+   && path && path[0])
+   {
+      if(is_file)
+      {
+	 dirname_modify(path);
+	 if(!path[0])
+	    strcpy(path,"~");
+      }
+      char *newcwd=string_alloca(xstrlen(path)+xstrlen(new_path)+2);
+      if(last_char(path)=='/')
+	 sprintf(newcwd,"%s%s",path,new_path);
+      else
+	 sprintf(newcwd,"%s/%s",path,new_path);
+      new_path=newcwd;
+   }
+   xfree(path);
+   path=xstrdup(new_path);
+   device_prefix_len=new_device_prefix_len;
+   Optimize();
+   strip_trailing_slashes(path);
+   is_file=new_is_file;
+   if(!strcmp(path,"/") || !strcmp(path,"//"))
+      is_file=false;
+}
+bool FileAccess::Path::operator==(const Path &p2) const
+{
+   const Path &p1=*this;
+   if(p1.is_file!=p2.is_file)
+      return false;
+   if(xstrcmp(p1.path,p2.path))
+      return false;
+   if(xstrcmp(p1.url,p2.url))
+      return false;
+   return true;
+}
+void FileAccess::Path::ExpandTilde(const Path &home)
+{
+   if(!home.path)
+      return;
+   if(path && path[0]=='~' && (path[1]=='/' || path[1]=='\0'))
+   {
+      device_prefix_len=home.device_prefix_len;
+      if(path[1]=='\0')
+	 is_file=home.is_file;
+   }
+   expand_tilde(&path,home.path);
+}
+
 #include "modconfig.h"
 #ifndef MODULE_PROTO_FTP
 # include "ftpclass.h"
