@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileAccess.h,v 1.81 2004/03/25 00:00:03 lav Exp $ */
+/* $Id: FileAccess.h,v 1.86 2005/08/04 13:10:40 lav Exp $ */
 
 #ifndef FILEACCESS_H
 #define FILEACCESS_H
@@ -77,14 +77,47 @@ public:
 
    struct fileinfo
    {
-      const char *file;
       off_t size;
+      const char *file;
       time_t time;
       bool get_size:1;
       bool get_time:1;
       bool is_dir:1;
    };
 
+   class Path
+   {
+      void init();
+   public:
+      int   device_prefix_len;
+      char  *path;
+      bool  is_file;
+      char  *url;
+      Path() { init(); }
+      Path(const Path *o) { init(); Set(o); }
+      Path(const Path &o) { init(); Set(o); }
+      Path(const char *new_path) { init(); Set(new_path); }
+      Path(const char *new_path,bool new_is_file=false,const char *new_url=0,int new_device_prefix_len=0)
+	 { init(); Set(new_path,new_is_file,new_url,new_device_prefix_len); }
+      ~Path();
+      void Set(const Path*);
+      void Set(const Path &o) { Set(&o); }
+      void Set(const char *new_path,bool new_is_file=false,const char *new_url=0,int device_prefix_len=0);
+      void SetURL(const char *u) { xfree(url); url=xstrdup(u); }
+      void Change(const char *new_path,bool new_is_file=false,const char *new_path_enc=0,int device_prefix_len=0);
+      void ExpandTilde(const Path &home);
+      static void Optimize(char *p,int dev_prefix=0);
+      void Optimize() { Optimize(path,device_prefix_len); }
+      const Path& operator=(const Path &o)
+	 {
+	    Set(&o);
+	    return *this;
+	 }
+      operator const char *() const { return path; }
+      bool operator==(const Path &p2) const;
+      bool operator!=(const Path &p2) const { return !(*this==p2); }
+   };
+
 protected:
    char	 *vproto;
    char	 *hostname;
@@ -93,9 +126,10 @@ protected:
    char  *pass;
    bool	 pass_open;
 
-   char	 *home;
    const char *default_cwd;
-   char  *cwd;
+   Path	 home;
+   Path	 cwd;
+   Path	 *new_cwd;
    char  *file;
    char	 *file_url;
    char	 *file1;
@@ -137,8 +171,7 @@ protected:
       }
    void set_home(const char *h)
       {
-	 xfree(home);
-	 home=xstrdup(h);
+	 home.Set(h);
 	 ExpandTildeInCWD();
       }
 
@@ -172,6 +205,9 @@ protected:
    char *suggested_filename;
    void SetSuggestedFileName(const char *fn);
 
+   char *entity_content_type;
+   char *entity_charset;
+
    FileAccess *next;
    static FileAccess *chain;
    FileAccess *FirstSameSite() { return NextSameSite(0); }
@@ -200,6 +236,7 @@ public:
 
    virtual void Connect(const char *h,const char *p);
    virtual void ConnectVerify();
+   void PathVerify(const Path &);
    virtual void AnonymousLogin();
    virtual void Login(const char *u,const char *p);
 
@@ -207,7 +244,7 @@ public:
    virtual void ResetLocationData();
 
    virtual void Open(const char *file,int mode,off_t pos=0);
-   void SetFileURL(const char *u) { xfree(file_url); file_url=xstrdup(u); }
+   void SetFileURL(const char *u);
    void SetSize(off_t s) { entity_size=s; }
    void SetDate(time_t d) { entity_date=d; }
    void WantDate(time_t *d) { opt_date=d; }
@@ -218,8 +255,8 @@ public:
    virtual void	Rename(const char *rfile,const char *to);
    virtual void Mkdir(const char *rfile,bool allpath=false);
    virtual void Chdir(const char *dir,bool verify=true);
-   void OptimizePath(char *path);
-   void SetCwd(const char *dir) { xfree(cwd); cwd=xstrdup(dir); }
+   void ChdirAccept() { cwd=*new_cwd; }
+   void SetCwd(const Path &new_cwd) { cwd=new_cwd; }
    void Remove(const char *rfile)    { Open(rfile,REMOVE); }
    void RemoveDir(const char *dir)  { Open(dir,REMOVE_DIR); }
    void Chmod(const char *file,int m);
@@ -244,7 +281,8 @@ public:
    void SeekReal() { pos=GetRealPos(); }
    void RereadManual() { norest_manual=true; }
 
-   const char *GetCwd() { return cwd; }
+   const Path& GetCwd() { return cwd; }
+   const Path& GetNewCwd() { return *new_cwd; }
    const char *GetFile() { return file; }
 
    virtual int Do() = 0;
@@ -302,6 +340,8 @@ public:
 
    const char *GetNewLocation() { return location; }
    const char *GetSuggestedFileName() { return suggested_filename; }
+   const char *GetEntityContentType() { return entity_content_type; }
+   const char *GetEntityCharset() { return entity_charset; }
 
    void Reconfig(const char *);
 
@@ -359,6 +399,7 @@ protected:
    bool done;
    char *error_text;
    void SetError(const char *);
+   void SetErrorCached(const char *);
 
    bool use_cache;
 
@@ -385,7 +426,7 @@ class ListInfo : public FileAccessOperation
 {
 protected:
    FileAccess *session;
-   char *saved_cwd;
+   FileAccess::Path saved_cwd;
    FileSet *result;
 
    const char *exclude_prefix;
