@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileCopy.cc,v 1.108 2005/04/04 09:59:39 lav Exp $ */
+/* $Id: FileCopy.cc,v 1.115 2005/06/22 08:04:17 lav Exp $ */
 
 /* FileCopyPeer behaviour:
     1) when suspended, does nothing
@@ -46,13 +46,14 @@
 #include "misc.h"
 #include "LsCache.h"
 #include "plural.h"
+#include "ArgV.h"
 
 #define skip_threshold 0x1000
 #define debug(a) Log::global->Format a
 
 ResDecl rate_period  ("xfer:rate-period","15", ResMgr::UNumberValidate,ResMgr::NoClosure);
 ResDecl eta_period   ("xfer:eta-period", "120",ResMgr::UNumberValidate,ResMgr::NoClosure);
-ResDecl max_redir    ("xfer:max-redirections", "0",ResMgr::UNumberValidate,ResMgr::NoClosure);
+ResDecl max_redir    ("xfer:max-redirections", "5",ResMgr::UNumberValidate,ResMgr::NoClosure);
 
 // FileCopy
 #define super SMTask
@@ -356,7 +357,7 @@ int FileCopy::Do()
       state=GET_DONE_WAIT;
       m=MOVED;
       end_time=now;
-      Delete(put); put=0;
+      put->Suspend();
       /* fallthrough */
    case(GET_DONE_WAIT):
       if(get->Error())
@@ -370,7 +371,7 @@ int FileCopy::Do()
 	 return m;
       debug((10,"copy: get is finished - all done\n"));
       state=ALL_DONE;
-      Delete(get); get=0;
+      get->Suspend();
       return MOVED;
 
    pre_GET_INFO_WAIT:
@@ -435,25 +436,15 @@ FileCopy *FileCopy::New(FileCopyPeer *s,FileCopyPeer *d,bool c)
       return res;
    return new FileCopy(s,d,c);
 }
-void FileCopy::Suspend()
+void FileCopy::SuspendInternal()
 {
-   if(get) get->Suspend();
-   if(put) put->Suspend();
-   super::Suspend();
+   if(get) get->SuspendSlave();
+   if(put) put->SuspendSlave();
 }
-void FileCopy::Resume()
+void FileCopy::ResumeInternal()
 {
-   super::Resume();
-   if(state!=PUT_WAIT)
-   {
-      if(get && !(put && put->Size()>=max_buf))
-	 get->Resume();
-   }
-   if(state!=GET_INFO_WAIT)
-   {
-      if(put)
-	 put->Resume();
-   }
+   if(get) get->ResumeSlave();
+   if(put) put->ResumeSlave();
 }
 void FileCopy::Fg()
 {
@@ -724,6 +715,19 @@ int FileCopyPeerFA::Do()
 
    if(Done() || Error())
       return m;
+
+   if(verify)
+   {
+      if(verify->Error())
+	 SetError(verify->ErrorText());
+      if(verify->Done())
+      {
+	 done=true;
+	 m=MOVED;
+      }
+      return m;
+   }
+
    if(want_size && size==NO_SIZE_YET && (mode==PUT || !start_transfer))
    {
       if(session->IsClosed())
@@ -781,8 +785,9 @@ int FileCopyPeerFA::Do()
 	    fxp_eof:
 	       // FIXME: set date for real.
 	       date_set=true;
-	       done=true;
-	       m=MOVED;
+	       if(!verify)
+		  verify=new FileVerificator(session,file);
+	       return MOVED;
 	    }
 	    else if(res==FA::IN_PROGRESS)
 	       return m;
@@ -843,21 +848,21 @@ bool FileCopyPeerFA::IOReady()
    return session->IOReady();
 }
 
-void FileCopyPeerFA::Suspend()
+void FileCopyPeerFA::SuspendInternal()
 {
    if(fxp && mode==PUT)
       return;
-   session->Suspend();
-   super::Suspend();
+   session->SuspendSlave();
 }
-void FileCopyPeerFA::Resume()
+void FileCopyPeerFA::ResumeInternal()
 {
-   super::Resume();
-   session->Resume();
+   session->ResumeSlave();
 }
 
 const char *FileCopyPeerFA::GetStatus()
 {
+   if(verify)
+      return verify->Status();
    if(!session->IsOpen())
       return 0;
    return session->CurrentStatus();
@@ -1155,6 +1160,7 @@ void FileCopyPeerFA::Init()
    redirections=0;
    can_seek=true;
    can_seek0=true;
+   verify=0;
 }
 
 FileCopyPeerFA::FileCopyPeerFA(FileAccess *s,const char *f,int m)
@@ -1178,6 +1184,7 @@ FileCopyPeerFA::~FileCopyPeerFA()
    }
    xfree(file);
    xfree(orig_url);
+   Delete(verify);
 }
 
 FileCopyPeerFA::FileCopyPeerFA(ParsedURL *u,int m)
@@ -1240,12 +1247,14 @@ FileCopyPeerFDStream::FileCopyPeerFDStream(FDStream *o,dir_t m)
    put_ll_timer=0;
    if(m==PUT)
       put_ll_timer=new Timer(TimeDiff(0,200));
+   verify=0;
 }
 FileCopyPeerFDStream::~FileCopyPeerFDStream()
 {
    if(delete_stream)
       delete stream;
    delete put_ll_timer;
+   Delete(verify);
 }
 
 void FileCopyPeerFDStream::Seek_LL()
@@ -1322,6 +1331,19 @@ int FileCopyPeerFDStream::Do()
    int m=STALL;
    if(Done() || Error())
       return m;
+
+   if(verify)
+   {
+      if(verify->Error())
+	 SetError(verify->ErrorText());
+      if(verify->Done())
+      {
+	 done=true;
+	 m=MOVED;
+      }
+      return m;
+   }
+
    switch(mode)
    {
    case PUT:
@@ -1340,7 +1362,8 @@ int FileCopyPeerFDStream::Do()
 	    }
 	    if(stream && delete_stream && !stream->Done())
 	       return m;
-	    done=true;
+	    if(!verify)
+	       verify=new FileVerificator(stream);
 	    return MOVED;
 	 }
 	 if(seek_pos==0)
@@ -1632,6 +1655,8 @@ void FileCopyPeerFDStream::RemoveFile()
 
 const char *FileCopyPeerFDStream::GetStatus()
 {
+   if(verify)
+      return verify->Status();
    return stream->status;
 }
 void FileCopyPeerFDStream::Kill(int sig)
@@ -1697,5 +1722,113 @@ int FileCopyPeerDirList::Do()
    return MOVED;
 }
 
+// FileVerificator
+void FileVerificator::Init0()
+{
+   done=false;
+   error_text=0;
+   verify_buffer=0;
+   verify_process=0;
+   if(!ResMgr::QueryBool("xfer:verify",0))
+      done=true;
+}
+void FileVerificator::InitVerify(const char *f)
+{
+   if(done)
+      return;
+   ArgV *args=new ArgV(ResMgr::Query("xfer:verify-command",0));
+   args->Append(f);
+   verify_process=new InputFilter(args);
+   verify_process->StderrToStdout();
+   verify_buffer=new IOBufferFDStream(verify_process,IOBuffer::GET);
+}
+FileVerificator::FileVerificator(const char *f)
+{
+   Init0();
+   InitVerify(f);
+}
+FileVerificator::FileVerificator(const FDStream *stream)
+{
+   Init0();
+   if(done)
+      return;
+   const char *f=stream->full_name;
+   if(!f)
+   {
+      done=true;
+      return;
+   }
+   const char *cwd=stream->GetCwd();
+   int cwd_len=xstrlen(cwd);
+   if(cwd && cwd_len>0 && !strncmp(f,cwd,cwd_len))
+   {
+      f+=cwd_len;
+      while(*f=='/')
+	 f++;
+      if(*f==0)
+	 f=".";
+   }
+   InitVerify(f);
+   if(verify_process)
+   {
+      verify_process->SetProcGroup(stream->GetProcGroup());
+      verify_process->SetCwd(cwd);
+   }
+}
+FileVerificator::FileVerificator(FileAccess *session,const char *f)
+{
+   Init0();
+   if(done)
+      return;
+   if(strcmp(session->GetProto(),"file"))
+   {
+      done=true;
+      return;
+   }
+   InitVerify(f);
+   verify_process->SetCwd(session->GetCwd());
+}
+FileVerificator::~FileVerificator()
+{
+   xfree(error_text);
+   // verify_process is deleted by verify_buffer dtor
+   Delete(verify_buffer);
+}
+int FileVerificator::Do()
+{
+   int m=STALL;
+   if(done)
+      return m;
+   verify_process->Kill(SIGCONT);
+   if(!verify_buffer->Eof())
+      return m;
+   if(verify_process->GetProcState()!=ProcWait::TERMINATED)
+      return m;
+   done=true;
+   m=MOVED;
+   if(verify_process->GetProcExitCode()!=0)
+   {
+      verify_buffer->Put("",1);
+      int len;
+      const char *s;
+      verify_buffer->Get(&s,&len);
+      len=strlen(s);
+      while(len>0 && s[len-1]=='\n')
+	 len--;
+      if(len==0)
+      {
+	 s=_("Verify command failed without a message");
+	 len=strlen(s);
+      }
+      error_text=xstrdup(s);
+      error_text[len]=0;
+      char *nl=strrchr(error_text,'\n');
+      if(nl)
+	 memmove(error_text,nl+1,len-(nl+1-error_text)+1);
+   }
+   return m;
+}
+
+
 // special pointer to creator of ftp/ftp copier. It is init'ed in Ftp class.
 FileCopy *(*FileCopy::fxp_create)(FileCopyPeer *src,FileCopyPeer *dst,bool cont);
