@@ -49,20 +49,12 @@ void Fish::GetBetterConnection(int level)
       {
 	 if(level<2)
 	    continue;
-	 if(!connection_takeover || (o->priority>=priority && !o->suspended))
+	 if(!connection_takeover || (o->priority>=priority && !o->IsSuspended()))
 	    continue;
 	 o->Disconnect();
 	 return;
       }
 
-      if(home && !o->home)
-	 o->home=xstrdup(home);
-      else if(!home && o->home)
-	 home=xstrdup(o->home);
-
-      o->ExpandTildeInCWD();
-      ExpandTildeInCWD();
-
       if(level==0 && xstrcmp(real_cwd,o->real_cwd))
 	 continue;
 
@@ -86,7 +78,7 @@ int Fish::Do()
 	 Disconnect();
 	 return m;
       }
-      TimeoutS(idle_start+idle-now);
+      TimeoutS(idle_start+idle-time_t(now));
    }
 
    if(Error())
@@ -146,7 +138,7 @@ int Fish::Do()
 
       const char *prog=Query("connect-program",hostname);
       if(!prog || !prog[0])
-	 prog="ssh -ax";
+	 prog="ssh -a -x";
       char *a=alloca_strdup(prog);
       ArgV *cmd=new ArgV;
       for(a=strtok(a," "); a; a=strtok(0," "))
@@ -204,7 +196,7 @@ int Fish::Do()
       Send("#VER 0.0.2\n"
 	   "echo '### 000'\n");
       PushExpect(EXPECT_VER);
-      if(home==0)
+      if(home_auto==0)
       {
 	 Send("#PWD\n"
 	      "pwd; echo '### 200'\n");
@@ -216,7 +208,7 @@ int Fish::Do()
    case CONNECTED:
       if(mode==CLOSED)
 	 return m;
-      if(home==0 && !RespQueueIsEmpty())
+      if(home.path==0 && !RespQueueIsEmpty())
 	 goto usual_return;
       ExpandTildeInCWD();
       if(mode!=CHANGE_DIR && xstrcmp(cwd,real_cwd))
@@ -224,7 +216,7 @@ int Fish::Do()
 	 if(path_queue_len==0 || strcmp(path_queue[path_queue_len-1],cwd))
 	 {
 	    Send("#CWD %s\n"
-		 "cd %s; echo '### 000'\n",cwd,shell_encode(cwd));
+		 "cd %s; echo '### 000'\n",cwd.path,shell_encode(cwd));
 	    PushExpect(EXPECT_CWD);
 	    PushDirectory(cwd);
 	 }
@@ -310,11 +302,14 @@ void Fish::MoveConnectionHere(Fish *o)
    RQ_alloc=o->RQ_alloc; o->RQ_alloc=0;
    RQ_head=o->RQ_head; o->RQ_head=0;
    RQ_tail=o->RQ_tail; o->RQ_tail=0;
-   event_time=o->event_time;
+   BumpEventTime(o->event_time);
    set_real_cwd(o->real_cwd);
    o->set_real_cwd(0);
    state=CONNECTED;
    o->Disconnect();
+   if(!home)
+      set_home(home_auto);
+   ResumeInternal();
 }
 
 void Fish::Disconnect()
@@ -331,6 +326,8 @@ void Fish::Disconnect()
       SetError(STORE_FAILED,0);
    received_greeting=false;
    password_sent=0;
+   xfree(home_auto); home_auto=0;
+   home_auto=xstrdup(FindHomeAuto());
 }
 
 void Fish::EmptyPathQueue()
@@ -345,7 +342,6 @@ void Fish::Init()
    state=DISCONNECTED;
    send_buf=0;
    recv_buf=0;
-   recv_buf_suspended=false;
    ssh=0;
    max_send=0;
    line=0;
@@ -534,22 +530,24 @@ void Fish::SendMethod()
    case RETRIEVE:
       if(pos>0)
       {
+	 int bs=0x1000;
+	 real_pos=pos-pos%bs;
 	 Send("#RETRP %lld %s\n"
 	      "ls -lLd %s; "
 	      "echo '### 100'; "
-	      "cat %s|(dd ibs=1 count=%lld of=/dev/null 2>/dev/null;cat); "
+	      "dd ibs=%d skip=%lld if=%s 2>/dev/null; "
 	      "echo '### 200'\n",
-	    (long long)pos,e,e,e,(long long)pos);
+	    (long long)real_pos,e,e,bs,(long long)real_pos/bs,e);
       }
       else
       {
 	 Send("#RETR %s\n"
 	   "ls -lLd %s; "
 	   "echo '### 100'; cat %s; echo '### 200'\n",e,e,e);
+	 real_pos=0;
       }
       PushExpect(EXPECT_RETR_INFO);
       PushExpect(EXPECT_RETR);
-      real_pos=pos;
       break;
    case STORE:
       if(entity_size<0)
@@ -750,7 +748,12 @@ int Fish::HandleReplies()
    case EXPECT_PWD:
       if(!message)
 	 break;
-      set_home(message);
+      home_auto=xstrdup(message);
+      Log::global->Format(9,"---- home set to %s\n",home_auto);
+      PropagateHomeAuto();
+      if(!home)
+	 set_home(home_auto);
+      LsCache::SetDirectory(this, home, true);
       break;
    case EXPECT_CWD:
       p=PopDirectory();
@@ -759,11 +762,10 @@ int Fish::HandleReplies()
 	 set_real_cwd(p);
 	 if(mode==CHANGE_DIR && RespQueueIsEmpty())
 	 {
-	    xfree(cwd);
-	    cwd=p;
-	    p=0;
+	    cwd.Set(p);
 	    eof=true;
 	 }
+	 LsCache::SetDirectory(this,p,true);
       }
       else
 	 SetError(NO_FILE,message);
@@ -1085,28 +1087,19 @@ int Fish::Done()
    return IN_PROGRESS;
 }
 
-void Fish::Suspend()
+void Fish::SuspendInternal()
 {
-   if(suspended)
-      return;
    if(recv_buf)
-   {
-      recv_buf_suspended=recv_buf->IsSuspended();
-      recv_buf->Suspend();
-   }
+      recv_buf->SuspendSlave();
    if(send_buf)
-      send_buf->Suspend();
-   super::Suspend();
+      send_buf->SuspendSlave();
 }
-void Fish::Resume()
+void Fish::ResumeInternal()
 {
-   if(!suspended)
-      return;
-   super::Resume();
-   if(recv_buf && !recv_buf_suspended)
-      recv_buf->Resume();
+   if(recv_buf)
+      recv_buf->ResumeSlave();
    if(send_buf)
-      send_buf->Resume();
+      send_buf->ResumeSlave();
 }
 
 const char *Fish::CurrentStatus()
@@ -1217,9 +1210,15 @@ int FishDirList::Do()
    {
       const char *cache_buffer=0;
       int cache_buffer_size=0;
-      if(use_cache && LsCache::Find(session,pattern,FA::LONG_LIST,
+      int err;
+      if(use_cache && LsCache::Find(session,pattern,FA::LONG_LIST,&err,
 				    &cache_buffer,&cache_buffer_size))
       {
+	 if(err)
+	 {
+	    SetErrorCached(cache_buffer);
+	    return MOVED;
+	 }
 	 ubuf=new IOBuffer(IOBuffer::GET);
 	 ubuf->Put(cache_buffer,cache_buffer_size);
 	 ubuf->PutEOF();
@@ -1240,8 +1239,7 @@ int FishDirList::Do()
    if(b==0) // eof
    {
       buf->PutEOF();
-      LsCache::Add(session,pattern,FA::LONG_LIST, ubuf);
-
+      LsCache::Add(session,pattern,FA::LONG_LIST,FA::OK,ubuf);
       return MOVED;
    }
 
@@ -1288,17 +1286,15 @@ const char *FishDirList::Status()
    return "";
 }
 
-void FishDirList::Suspend()
+void FishDirList::SuspendInternal()
 {
    if(ubuf)
-      ubuf->Suspend();
-   super::Suspend();
+      ubuf->SuspendSlave();
 }
-void FishDirList::Resume()
+void FishDirList::ResumeInternal()
 {
-   super::Resume();
    if(ubuf)
-      ubuf->Resume();
+      ubuf->ResumeSlave();
 }
 
 static FileSet *ls_to_FileSet(const char *b,int len)
