@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FtpDirList.cc,v 1.22 2004/03/24 22:27:30 lav Exp $ */
+/* $Id: FtpDirList.cc,v 1.24 2005/06/22 08:04:19 lav Exp $ */
 
 #include <config.h>
 
@@ -55,9 +55,15 @@ int FtpDirList::Do()
    {
       const char *cache_buffer=0;
       int cache_buffer_size=0;
-      if(use_cache && LsCache::Find(session,pattern,FA::LONG_LIST,
+      int err;
+      if(use_cache && LsCache::Find(session,pattern,FA::LONG_LIST,&err,
 				    &cache_buffer,&cache_buffer_size))
       {
+	 if(err)
+	 {
+	    SetErrorCached(cache_buffer);
+	    return MOVED;
+	 }
 	 ubuf=new IOBuffer(IOBuffer::GET);
 	 ubuf->Put(cache_buffer,cache_buffer_size);
 	 ubuf->PutEOF();
@@ -77,8 +83,7 @@ int FtpDirList::Do()
    if(b==0) // eof
    {
       buf->PutEOF();
-      LsCache::Add(session,pattern,FA::LONG_LIST, ubuf);
-
+      LsCache::Add(session,pattern,FA::LONG_LIST,FA::OK,ubuf);
       return MOVED;
    }
 
@@ -140,17 +145,15 @@ const char *FtpDirList::Status()
    return "";
 }
 
-void FtpDirList::Suspend()
+void FtpDirList::SuspendInternal()
 {
    if(ubuf)
-      ubuf->Suspend();
-   super::Suspend();
+      ubuf->SuspendSlave();
 }
-void FtpDirList::Resume()
+void FtpDirList::ResumeInternal()
 {
-   super::Resume();
    if(ubuf)
-      ubuf->Resume();
+      ubuf->ResumeSlave();
 }
 
 void FtpDirList::FormatGeneric(FileInfo *fi)
