@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Http.cc,v 1.194 2005/05/18 05:57:08 lav Exp $ */
+/* $Id: Http.cc,v 1.199 2005/08/04 10:10:29 lav Exp $ */
 
 #include <config.h>
 #include "trio.h"
@@ -70,7 +70,6 @@ void Http::Init()
    sock=-1;
    send_buf=0;
    recv_buf=0;
-   recv_buf_suspended=false;
    body_size=-1;
    bytes_received=0;
    line=0;
@@ -140,9 +139,10 @@ void Http::MoveConnectionHere(Http *o)
    sock=o->sock; o->sock=-1;
    rate_limit=o->rate_limit; o->rate_limit=0;
    last_method=o->last_method; o->last_method=0;
-   event_time=o->event_time;
+   BumpEventTime(o->event_time);
    state=CONNECTED;
    o->Disconnect();
+   ResumeInternal();
 }
 
 void Http::Disconnect()
@@ -334,7 +334,7 @@ void Http::SendMethod(const char *method,const char *efile)
       if(!xstrcmp(referer,"."))
       {
 	 referer=GetConnectURL(NO_USER+NO_PASSWORD);
-	 if(last_char(referer)!='/')
+	 if(last_char(referer)!='/' && !cwd.is_file)
 	    slash="/";
       }
       if(referer && referer[0])
@@ -440,12 +440,14 @@ void Http::DirFile(char *path_base,const char *ecwd,const char *efile)
       strcpy(path_base,efile);
    else if(efile[0]=='~')
       sprintf(path_base,"/%s",efile);
-   else if(cwd[0]==0 || ((cwd[0]=='/' || (!hftp && cwd[0]=='~')) && cwd[1]==0))
+   else if(ecwd[0]==0 || ((ecwd[0]=='/' || (!hftp && ecwd[0]=='~')) && ecwd[1]==0))
       sprintf(path_base,"/%s",efile);
-   else if(cwd[0]=='~')
+   else if(ecwd[0]=='~' && efile[0])
       sprintf(path_base,"/%s/%s",ecwd,efile);
-   else
+   else if(efile[0])
       sprintf(path_base,"%s/%s",ecwd,efile);
+   else
+      strcpy(path_base,ecwd);
 
    if(path_base[1]=='~' && path_base[2]=='/')
       memmove(path_base,path_base+2,strlen(path_base+2)+1);
@@ -461,10 +463,30 @@ void Http::SendRequest(const char *connection,const char *f)
 {
    char *efile=string_alloca(strlen(f)*3+1);
    url::encode_string(f,efile,URL_PATH_UNSAFE);
-   char *ecwd=string_alloca(strlen(cwd)*3+1);
-   url::encode_string(cwd,ecwd,URL_PATH_UNSAFE);
+   char *ecwd;
    int efile_len;
 
+   if(cwd.url)
+   {
+      ecwd=cwd.url+url::path_index(cwd.url);
+      ecwd=alloca_strdup(ecwd);
+   }
+   else
+   {
+      ecwd=string_alloca(strlen(cwd)*3+1);
+      url::encode_string(cwd,ecwd,URL_PATH_UNSAFE);
+   }
+   bool add_slash=true;
+   if(cwd.is_file)
+   {
+      if(efile[0])
+	 dirname_modify(ecwd+(!strncmp(ecwd,"/~",2)));
+      else
+	 add_slash=false;
+   }
+   if(mode==CHANGE_DIR && new_cwd)
+      add_slash=!new_cwd->is_file;
+
    char *pfile=string_alloca(4+3+xstrlen(user)*6+3+xstrlen(pass)*3+1+
 			      strlen(hostname)*3+1+xstrlen(portname)*3+1+
 			      strlen(ecwd)+1+strlen(efile)+1+6+1);
@@ -585,7 +607,7 @@ void Http::SendRequest(const char *connection,const char *f)
    case LONG_LIST:
    case MP_LIST:
    case MAKE_DIR:
-      if(efile_len<1 || efile[efile_len-1]!='/')
+      if(last_char(efile)!='/' && add_slash)
 	 strcat(efile,"/");
       if(mode==CHANGE_DIR)
 	 SendMethod("HEAD",efile);
@@ -874,7 +896,7 @@ void Http::GetBetterConnection(int level)
       {
 	 if(level<2)
 	    continue;
-	 if(!connection_takeover || (o->priority>=priority && !o->suspended))
+	 if(!connection_takeover || (o->priority>=priority && !o->IsSuspended()))
 	    continue;
 	 o->Disconnect();
 	 return;
@@ -914,8 +936,8 @@ int Http::Do()
       TimeoutS(idle_start+idle-time_t(now));
    }
 
-   if(home==0)
-      home=xstrdup(default_cwd);
+   if(home.path==0)
+      home.Set(default_cwd);
    ExpandTildeInCWD();
 
    if(Error())
@@ -1466,7 +1488,7 @@ int Http::Do()
 	    if(closure && closure[0])
 	       sprintf(err,"%s (%s)",status+status_consumed,closure);
 	    else
-	       sprintf(err,"%s (%s%s)",status+status_consumed,cwd,
+	       sprintf(err,"%s (%s%s)",status+status_consumed,cwd.path,
 				       (last_char(cwd)=='/')?"":"/");
 	 }
 	 state=RECEIVING_BODY;
@@ -1475,8 +1497,7 @@ int Http::Do()
       }
       if(mode==CHANGE_DIR)
       {
-	 xfree(cwd);
-	 cwd=xstrdup(file);
+	 cwd.Set(new_cwd);
 	 state=DONE;
 	 return MOVED;
       }
@@ -1562,28 +1583,19 @@ void  Http::ClassInit()
 #endif
 }
 
-void Http::Suspend()
+void Http::SuspendInternal()
 {
-   if(suspended)
-      return;
    if(recv_buf)
-   {
-      recv_buf_suspended=recv_buf->IsSuspended();
-      recv_buf->Suspend();
-   }
+      recv_buf->SuspendSlave();
    if(send_buf)
-      send_buf->Suspend();
-   super::Suspend();
+      send_buf->SuspendSlave();
 }
-void Http::Resume()
+void Http::ResumeInternal()
 {
-   if(!suspended)
-      return;
-   super::Resume();
-   if(recv_buf && !recv_buf_suspended)
-      recv_buf->Resume();
+   if(recv_buf)
+      recv_buf->ResumeSlave();
    if(send_buf)
-      send_buf->Resume();
+      send_buf->ResumeSlave();
 }
 
 int Http::Read(void *buf,int size)
@@ -1904,7 +1916,7 @@ bool Http::SameLocationAs(FileAccess *fa)
    if(!SameSiteAs(fa))
       return false;
    Http *o=(Http*)fa;
-   if(xstrcmp(cwd,o->cwd))
+   if(cwd!=o->cwd)
       return false;
    return true;
 }
@@ -2181,10 +2193,8 @@ void HFtp::Login(const char *u,const char *p)
    super::Login(u,p);
    if(u)
    {
-      xfree(home);
-      home=xstrdup("~");
-      xfree(cwd);
-      cwd=xstrdup(home);
+      home.Set("~");
+      cwd.Set(home,false,0);
    }
 }
 void HFtp::Reconfig(const char *name)
