@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Http.h,v 1.38.2.1 2004/01/23 19:17:01 lav Exp $ */
+/* $Id: Http.h,v 1.46 2005/06/22 08:04:19 lav Exp $ */
 
 #ifndef HTTP_H
 #define HTTP_H
@@ -53,12 +53,14 @@ class Http : public NetAccess
 
    IOBuffer *send_buf;
    IOBuffer *recv_buf;
-   bool recv_buf_suspended;
    void SendMethod(const char *,const char *);
    const char *last_method;
-   bool post;
-   char *post_data;
+   enum { HTTP_NONE=0, HTTP_POST, HTTP_MOVE, HTTP_COPY } special;
+   char *special_data;
+   void DirFile(char *path_base,const char *ecwd,const char *efile);
    void SendAuth();
+   void SendCacheControl();
+   void SendBasicAuth(const char *tag,const char *auth);
    void SendBasicAuth(const char *tag,const char *u,const char *p);
    void SendRequest(const char *connection,const char *f);
    void SendRequest(const char *connection=0)
@@ -160,11 +162,13 @@ public:
 
    bool NeedSizeDateBeforehand() { return true; }
 
-   void Suspend();
-   void Resume();
+   void SuspendInternal();
+   void ResumeInternal();
 
    void Cleanup();
    void CleanupThis();
+
+   static time_t atotm (const char *time_string);
 };
 
 class HFtp : public Http
