@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: HttpDir.cc,v 1.86 2005/04/29 05:42:39 lav Exp $ */
+/* $Id: HttpDir.cc,v 1.88 2005/08/09 06:40:40 lav Exp $ */
 
 #include <config.h>
 #include "ascii_ctype.h"
@@ -1261,6 +1261,11 @@ int HttpDirList::Do()
       }
       else
       {
+	 if(mode==FA::MP_LIST && !*curr && session->GetCwd().is_file)
+	 {
+	    mode=FA::LONG_LIST;
+	    goto retry;
+	 }
 	 session->Open(curr,mode);
 	 session->UseCache(use_cache);
 	 ubuf=new IOBufferFileAccess(session);
@@ -1378,17 +1383,15 @@ const char *HttpDirList::Status()
    return "";
 }
 
-void HttpDirList::Suspend()
+void HttpDirList::SuspendInternal()
 {
    if(ubuf)
-      ubuf->Suspend();
-   super::Suspend();
+      ubuf->SuspendSlave();
 }
-void HttpDirList::Resume()
+void HttpDirList::ResumeInternal()
 {
-   super::Resume();
    if(ubuf)
-      ubuf->Resume();
+      ubuf->ResumeSlave();
 }
 
 
