@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Job.h,v 1.20 2002/08/29 07:26:19 lav Exp $ */
+/* $Id: Job.h,v 1.21 2005/06/22 09:26:00 lav Exp $ */
 
 #ifndef JOB_H
 #define JOB_H
@@ -126,6 +126,9 @@ public:
    virtual const char *GetConnectURL() { return 0; }
    virtual void lftpMovesToBackground() { Resume(); }
    static  void lftpMovesToBackground_ToAll();
+
+   virtual off_t GetBytesCount() { return 0; }
+   virtual double GetTimeSpent() { return 0; }
 };
 
 class SessionJob : public Job
