@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: LsCache.cc,v 1.21 2003/06/26 10:12:59 lav Exp $ */
+/* $Id: LsCache.cc,v 1.30 2005/08/03 07:12:47 lav Exp $ */
 
 #include <config.h>
 #include <assert.h>
@@ -31,56 +31,56 @@ LsCache *LsCache::chain=0;
 bool	 LsCache::use=true;
 long	 LsCache::sizelimit=1024*1024;
 TimeInterval LsCache::ttl("60m");  // time to live = 60 minutes
+TimeInterval LsCache::ttl_neg("1m");  // time to live for negative cache
 LsCache::ExpireHelper LsCache::expire_helper;
 
-FileSet *LsCache::fset=0;
-FileAccess *LsCache::fset_loc=0;
-int LsCache::fset_m=0;
-char *LsCache::fset_a=0;
-
 void LsCache::CheckSize()
 {
    if(sizelimit<0)
       return;  // no limit
 
    LsCache **oldest;
-   time_t   oldest_time;
+   time_t   oldest_expire_time;
    LsCache **scan;
 
    for(;;)
    {
       long size=0;
       oldest=&chain;
-      oldest_time=0;
+      oldest_expire_time=0;
       for(scan=&chain; *scan; scan=&(*scan)->next)
       {
-	 if(*oldest || oldest_time<(*scan)->timestamp)
+	 time_t e_time=(*scan)->ExpireTime();
+	 if(oldest_expire_time<e_time)
 	 {
 	    oldest=scan;
-	    oldest_time=(*scan)->timestamp;
+	    oldest_expire_time=e_time;
 	 }
 	 size+=scan[0]->data_len;
+	 if(scan[0]->afset)
+	    size+=scan[0]->afset->EstimateMemory();
       }
       if(size<=sizelimit)
 	 break;
-      LsCache *tmp=oldest[0]->next;
-      delete(*oldest);
-      *oldest=tmp;
+      delete replace_value(oldest[0],oldest[0]->next);
    }
 }
 
 ResDecl res_cache_empty_listings("cache:cache-empty-listings","no",ResMgr::BoolValidate,0);
 ResDecl res_cache_enable("cache:enable","yes",ResMgr::BoolValidate,0);
 ResDecl res_cache_expire("cache:expire","60m",ResMgr::TimeIntervalValidate,0);
+ResDecl res_cache_expire_neg("cache:expire-negative","1m",ResMgr::TimeIntervalValidate,0);
 ResDecl res_cache_size  ("cache:size","1048576",ResMgr::UNumberValidate,ResMgr::NoClosure);
 
-void LsCache::Add(FileAccess *p_loc,const char *a,int m,const char *d,int l)
+void LsCache::Add(FileAccess *p_loc,const char *a,int m,int e,const char *d,int l,const FileSet *fs)
 {
    if(!strcmp(p_loc->GetProto(),"file"))
       return;  // don't cache local objects
    if(l == 0 &&
 	 !res_cache_empty_listings.QueryBool(p_loc->GetHostName()))
       return;
+   if(e!=FA::OK && e!=FA::NO_FILE && e!=FA::NOT_SUPP)
+      return;
 
    CheckSize();
 
@@ -105,35 +105,41 @@ void LsCache::Add(FileAccess *p_loc,const char *a,int m,const char *d,int l)
    else
    {
       xfree(scan->data);
-      if(fset_loc && scan->loc->SameLocationAs(fset_loc))
-	 free_fset();
+      delete scan->afset;
    }
+   scan->err_code=e;
    scan->data=(char*)xmemdup(d,l);
    scan->data_len=l;
-   time(&scan->timestamp);
-   if(expire_helper.expiring==0)
+   scan->afset=fs?new FileSet(fs):0;
+   scan->timestamp=SMTask::now;
+   if(expire_helper.expiring==0
+   || scan->ExpireTime()<expire_helper.expiring->ExpireTime())
       expire_helper.expiring=scan;
-   return;
 }
 
-void LsCache::Add(FileAccess *p_loc,const char *a,int m,const Buffer *ubuf)
+void LsCache::Add(FileAccess *p_loc,const char *a,int m,int e,const Buffer *ubuf,const FileSet *fs)
 {
    if(!ubuf->IsSaving())
       return;
 
    const char *cache_buffer;
    int cache_buffer_size;
-   ubuf->GetSaved(&cache_buffer,&cache_buffer_size);
-   LsCache::Add(p_loc,a,m,cache_buffer,cache_buffer_size);
+   if(e)
+   {
+      cache_buffer=ubuf->ErrorText();
+      cache_buffer_size=strlen(cache_buffer)+1;
+   }
+   else
+      ubuf->GetSaved(&cache_buffer,&cache_buffer_size);
+   LsCache::Add(p_loc,a,m,e,cache_buffer,cache_buffer_size,fs);
 }
 
-int LsCache::Find(FileAccess *p_loc,const char *a,int m,const char **d,int *l)
+int LsCache::Find(FileAccess *p_loc,const char *a,int m,int *e,const char **d,int *l,FileSet **fs)
 {
    if(!ResMgr::QueryBool("cache:enable",p_loc->GetHostName()))
       return 0;
 
-   LsCache *scan;
-   for(scan=chain; scan; scan=scan->next)
+   for(LsCache *scan=chain; scan; scan=scan->next)
    {
       if((m == -1 || scan->mode==m) && !strcmp(scan->arg,a) && p_loc->SameLocationAs(scan->loc))
       {
@@ -142,6 +148,9 @@ int LsCache::Find(FileAccess *p_loc,const char *a,int m,const char **d,int *l)
 	    *d=scan->data;
 	    *l=scan->data_len;
 	 }
+	 if(fs)
+	    *fs=scan->afset;
+	 *e=scan->err_code;
 	 return 1;
       }
    }
@@ -150,48 +159,36 @@ int LsCache::Find(FileAccess *p_loc,const char *a,int m,const char **d,int *l)
 
 FileSet *LsCache::FindFileSet(FileAccess *p_loc,const char *a,int m)
 {
-   if(fset && m==fset_m && !strcmp(fset_a,a) && p_loc->SameLocationAs(fset_loc))
-      return fset;
-
    const char *buf_c;
    int bufsiz;
-   if(!Find(p_loc, a, m, &buf_c, &bufsiz))
+   int e;
+   FileSet *fs=0;
+   if(!Find(p_loc, a, m, &e, &buf_c, &bufsiz, &fs))
       return 0;
 
-   free_fset();
+   if(fs)
+      return fs;
 
-   FileSet *new_fset=p_loc->ParseLongList(buf_c, bufsiz);
-   if(!new_fset)
+   fs=p_loc->ParseLongList(buf_c, bufsiz);
+   if(!fs)
       return 0;
 
-   fset=new_fset;
-   fset_a=xstrdup(a);
-   fset_m=m;
-   fset_loc=p_loc->Clone();
-
-   return fset;
-}
-
-void LsCache::free_fset()
-{
-   SMTask::Delete(fset_loc);
-   xfree(fset_a);
-   delete fset;
-
-   fset_loc=0;
-   fset_a=0;
-   fset=0;
+   for(LsCache *scan=chain; scan; scan=scan->next)
+   {
+      if(scan->data==buf_c)
+	 scan->afset=fs;
+   }
+   return fs;
 }
 
 LsCache::~LsCache()
 {
    if(expire_helper.expiring==this)
       expire_helper.expiring=0;
-   if(fset_loc && loc->SameLocationAs(fset_loc))
-      free_fset();
    SMTask::Delete(loc);
    xfree(data);
    xfree(arg);
+   delete afset;
 }
 
 void LsCache::Flush()
@@ -213,7 +210,7 @@ void LsCache::List()
 
    long vol=0;
    for(LsCache *scan=chain; scan; scan=scan->next)
-      vol+=scan->data_len;
+      vol+=scan->data_len+(scan->afset?scan->afset->EstimateMemory():0);
 
    printf(plural("%ld $#l#byte|bytes$ cached",vol),vol);
 
@@ -235,39 +232,61 @@ void LsCache::List()
    }
 }
 
+time_t LsCache::ExpireTime()
+{
+   TimeInterval *this_ttl=(err_code==FA::OK?&ttl:&ttl_neg);
+   if(this_ttl->IsInfty() || this_ttl->Seconds()==0)
+      return 0;
+   return(timestamp+this_ttl->Seconds());
+}
+bool LsCache::Expired()
+{
+   time_t e_time=ExpireTime();
+   return(e_time && e_time<=(time_t)SMTask::now);
+}
+
 int LsCache::ExpireHelper::Do()
 {
-   if(ttl.IsInfty() || ttl.Seconds()==0)
+   if((ttl.IsInfty() || ttl.Seconds()==0)
+   && (ttl_neg.IsInfty() || ttl_neg.Seconds()==0))
       return STALL;
-   if(!expiring || expiring->timestamp+ttl.Seconds() <= now)
+   if(!expiring || expiring->Expired())
    {
       LsCache **scan=&LsCache::chain;
       while(*scan)
       {
-	 if((*scan)->timestamp+ttl.Seconds() <= now)
+	 if((*scan)->Expired())
 	 {
-	    LsCache *tmp=*scan;
-	    *scan=tmp->next;
-	    delete tmp;
+	    delete replace_value(scan[0],scan[0]->next);
+	    scan=&LsCache::chain;   // rescan the chain to find next expiring
 	    continue;
 	 }
-	 if(!expiring || expiring->timestamp > (*scan)->timestamp)
+	 if(!expiring)
 	    expiring=*scan;
+	 else
+	 {
+	    // compare expire times and find the least one
+   	    time_t expiring_e_time=expiring->ExpireTime();
+	    time_t scan_e_time=(*scan)->ExpireTime();
+	    if(scan_e_time!=0 && scan_e_time!=0 && expiring_e_time > scan_e_time);
+	       expiring=*scan;
+	 }
 	 scan=&scan[0]->next;
       }
       if(!expiring)
 	 return STALL;
    }
-   time_t t_out=expiring->timestamp+ttl.Seconds()-now;
+   time_t t_out=expiring->ExpireTime()-(time_t)now;
    if(t_out>1024)
       t_out=1024;
-   Timeout(t_out*1000);
+   TimeoutS(t_out);
    return STALL;
 }
 void LsCache::ExpireHelper::Reconfig(const char *name)
 {
    SetSizeLimit(ResMgr::Query("cache:size",0));
-   SetExpire(TimeInterval((const char*)ResMgr::Query("cache:expire",0)));
+   SetExpire(TimeInterval(ResMgr::Query("cache:expire",0)));
+   SetExpireNegative(TimeInterval(ResMgr::Query("cache:expire-negative",0)));
    use=ResMgr::QueryBool("cache:enable",0);
 }
 
@@ -276,15 +295,7 @@ void LsCache::Changed(change_mode m,FileAccess *f,const char *dir)
    const char *fdir_c=dir_file(f->GetCwd(),dir);
    char *fdir=alloca_strdup(fdir_c);
    if(m==FILE_CHANGED)
-   {
-      char *slash=strrchr(fdir,'/');
-      if(!slash)
-	 fdir[0]=0;
-      else if(slash>fdir)
-	 *slash=0;
-      else
-	 fdir[1]=0;
-   }
+      dirname_modify(fdir);
    int fdir_len=strlen(fdir);
 
    LsCache **scan=&LsCache::chain;
@@ -296,9 +307,7 @@ void LsCache::Changed(change_mode m,FileAccess *f,const char *dir)
 		     !strncmp(fdir,dir_file(sloc->GetCwd(),(*scan)->arg),fdir_len)
 		   : !strcmp (fdir,dir_file(sloc->GetCwd(),(*scan)->arg)))))
       {
-	 LsCache *tmp=*scan;
-	 *scan=tmp->next;
-	 delete tmp;
+	 delete replace_value(scan[0],scan[0]->next);
 	 continue;
       }
       scan=&scan[0]->next;
@@ -312,11 +321,11 @@ void LsCache::SetDirectory(FileAccess *p_loc, const char *path, bool dir)
    if(!path)
       return;
 
-   char *origdir = alloca_strdup(p_loc->GetCwd());
+   FileAccess::Path origdir = p_loc->GetCwd();
 
    p_loc->Chdir(path,false);
    const char *entry = dir? "1":"0";
-   LsCache::Add(p_loc,"",FileAccess::CHANGE_DIR, entry, strlen(entry));
+   LsCache::Add(p_loc,"",FileAccess::CHANGE_DIR, dir?FA::OK:FA::NO_FILE, entry, strlen(entry));
    p_loc->SetCwd(origdir);
 }
 
@@ -326,10 +335,7 @@ void LsCache::SetDirectory(FileAccess *p_loc, const char *path, bool dir)
 
 int LsCache::IsDirectory(FileAccess *p_loc,const char *dir_c)
 {
-   if(*dir_c && dir_c[strlen(dir_c)-1] == '/')
-      return 1;
-
-   char *origdir = alloca_strdup(p_loc->GetCwd());
+   FileAccess::Path origdir = p_loc->GetCwd();
    p_loc->Chdir(dir_c, false);
 
    int ret = -1;
@@ -340,18 +346,30 @@ int LsCache::IsDirectory(FileAccess *p_loc,const char *dir_c)
     * CHANGE_DIR entry for it. */
    const char *buf_c;
    int bufsiz;
-   if(Find(p_loc, "", FileAccess::CHANGE_DIR, &buf_c,&bufsiz))
+   int e;
+   if(Find(p_loc, "", FileAccess::CHANGE_DIR, &e, &buf_c,&bufsiz))
    {
-      ret = (buf_c[0]=='1');
+      assert(bufsiz==1);
+      ret = (e==FA::OK);
       goto leave;
    }
 
    /* We know the path is a directory if we have a cache entry for it.  This is
     * true regardless of the list type.  (Unless it's a CHANGE_DIR entry; do this
     * test after the CHANGE_DIR check.) */
-   if(Find(p_loc, "", -1, 0,0))
+   if(Find(p_loc, "", FA::LONG_LIST, &e, 0,0))
    {
-      ret = 1;
+      ret = (e==FA::OK);
+      goto leave;
+   }
+   if(Find(p_loc, "", FA::MP_LIST, &e, 0,0))
+   {
+      ret = (e==FA::OK);
+      goto leave;
+   }
+   if(Find(p_loc, "", FA::LIST, &e, 0,0))
+   {
+      ret = (e==FA::OK);
       goto leave;
    }
 
@@ -360,17 +378,12 @@ int LsCache::IsDirectory(FileAccess *p_loc,const char *dir_c)
    {
       p_loc->SetCwd(origdir);
       char *dir = alloca_strdup(dir_c);
-      char *sl = strrchr(dir, '/');
-      if(sl)
-      {
-	 if(sl>dir)
-	    *sl=0;
-	 else
-	    sl[1]=0;
-	 p_loc->Chdir(dir, false);
-      }
+      p_loc->Chdir(dirname_modify(dir), false);
 
-      const FileSet *fs=FindFileSet(p_loc, "", FA::LONG_LIST);
+
+      const FileSet *fs=FindFileSet(p_loc, "", FA::MP_LIST);
+      if(!fs)
+	 fs=FindFileSet(p_loc, "", FA::LONG_LIST);
       if(fs)
       {
 	 FileInfo *fi=fs->FindByName(basename_ptr(dir_c));
