@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: MirrorJob.cc,v 1.121 2005/05/13 13:33:00 lav Exp $ */
+/* $Id: MirrorJob.cc,v 1.126 2005/08/05 08:54:22 lav Exp $ */
 
 #include <config.h>
 
@@ -27,6 +27,7 @@
 #include <errno.h>
 #include <unistd.h>
 #include <assert.h>
+#include <mbswidth.h>
 #include "MirrorJob.h"
 #include "CmdExec.h"
 #include "rmJob.h"
@@ -104,17 +105,22 @@ final:
       printf(plural("%sModified: %d file$|s$, %d symlink$|s$\n",
 		     stats.mod_files,stats.mod_symlinks),
 	 tab,stats.mod_files,stats.mod_symlinks);
+   if(stats.bytes)
+      printf("%s%s\n",tab,CopyJob::FormatBytesTimeRate(stats.bytes,stats.time));
    if(stats.del_dirs || stats.del_files || stats.del_symlinks)
       printf(plural(flags&DELETE ?
 	       "%sRemoved: %d director$y|ies$, %d file$|s$, %d symlink$|s$\n"
 	      :"%sTo be removed: %d director$y|ies$, %d file$|s$, %d symlink$|s$\n",
 	      stats.del_dirs,stats.del_files,stats.del_symlinks),
 	 tab,stats.del_dirs,stats.del_files,stats.del_symlinks);
-   return;
+   if(stats.error_count)
+      printf(plural("%s%d error$|s$ detected\n",stats.error_count),
+	       tab,stats.error_count);
 }
 
 void  MirrorJob::ShowRunStatus(StatusLine *s)
 {
+   int w=s->GetWidthDelayed();
    switch(state)
    {
    case(INITIAL_STATE):
@@ -145,19 +151,27 @@ void  MirrorJob::ShowRunStatus(StatusLine *s)
    case(GETTING_LIST_INFO):
       if(target_list_info && (!source_list_info || now%4>=2))
       {
+	 const char *status=target_list_info->Status();
+	 int status_w=mbswidth(status, MBSW_ACCEPT_INVALID|MBSW_ACCEPT_UNPRINTABLE);
+	 int dw=w-status_w;
+	 if(dw<20)
+	    dw=20;
 	 if(target_relative_dir)
-	    s->Show("%s: %s",squeeze_file_name(target_relative_dir,20),
-	       target_list_info->Status());
+	    s->Show("%s: %s",squeeze_file_name(target_relative_dir,dw),status);
 	 else
-	    s->Show("%s",target_list_info->Status());
+	    s->Show("%s",status);
       }
       else if(source_list_info)
       {
+	 const char *status=source_list_info->Status();
+	 int status_w=mbswidth(status, MBSW_ACCEPT_INVALID|MBSW_ACCEPT_UNPRINTABLE);
+	 int dw=w-status_w;
+	 if(dw<20)
+	    dw=20;
 	 if(source_relative_dir)
-	    s->Show("%s: %s",squeeze_file_name(source_relative_dir,20),
-	       source_list_info->Status());
+	    s->Show("%s: %s",squeeze_file_name(source_relative_dir,dw),status);
 	 else
-	    s->Show("%s",source_list_info->Status());
+	    s->Show("%s",status);
       }
       break;
    }
@@ -565,7 +579,12 @@ void MirrorJob::HandleListInfoCreation(FileAccess * &session,ListInfo * &list_in
       return;
    }
    list_info->UseCache(use_cache);
-   list_info->Need(FileInfo::ALL_INFO);
+   int need=FileInfo::ALL_INFO;
+   if(flags&IGNORE_TIME)
+      need&=~FileInfo::DATE;
+   if(flags&IGNORE_SIZE)
+      need&=~FileInfo::SIZE;
+   list_info->Need(need);
    if(flags&RETR_SYMLINKS)
       list_info->FollowSymlinks();
 
@@ -735,6 +754,8 @@ int   MirrorJob::Do()
    case(WAITING_FOR_TRANSFER):
       while((j=FindDoneAwaitedJob())!=0)
       {
+	 stats.bytes+=j->GetBytesCount();
+	 stats.time +=j->GetTimeSpent();
 	 if(j->ExitCode()!=0)
 	    stats.error_count++;
 	 RemoveWaiting(j);
@@ -941,7 +962,7 @@ int   MirrorJob::Do()
 	 parent_mirror->stats.Add(stats);
       else
       {
-	 if((flags&LOOP) && stats.HaveSomethingDone() && !stats.error_count)
+	 if((flags&LOOP) && stats.HaveSomethingDone(flags) && !stats.error_count)
 	 {
 	    set_state(DONE);
 	    PrintStatus(0,"");
@@ -1061,7 +1082,7 @@ void MirrorJob::va_Report(const char *fmt,va_list v)
    if(verbose_report)
    {
       pid_t p=tcgetpgrp(fileno(stdout));
-      if(p!=-1 && p!=getpgrp())
+      if(p>0 && p!=getpgrp())
 	 return;
 
       vfprintf(stdout,fmt,v);
@@ -1162,9 +1183,10 @@ void MirrorJob::Statistics::Add(const Statistics &s)
    bytes       +=s.bytes;
    time	       +=s.time;
 }
-bool MirrorJob::Statistics::HaveSomethingDone()
+bool MirrorJob::Statistics::HaveSomethingDone(int flags)
 {
-   return new_files|mod_files|del_files|new_symlinks|mod_symlinks|del_symlinks|del_dirs;
+   bool del=(flags&MirrorJob::DELETE);
+   return new_files|mod_files|(del_files*del)|new_symlinks|mod_symlinks|(del_symlinks*del)|(del_dirs*del);
 }
 
 char *MirrorJob::SetScriptFile(const char *n)
