@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: MirrorJob.h,v 1.40 2004/10/28 06:23:34 lav Exp $ */
+/* $Id: MirrorJob.h,v 1.42 2005/08/05 08:54:22 lav Exp $ */
 
 #ifndef MIRRORJOB_H
 #define MIRRORJOB_H
@@ -80,8 +80,12 @@ class MirrorJob : public Job
       int dirs,del_dirs;
       int tot_symlinks,new_symlinks,mod_symlinks,del_symlinks;
       int error_count;
+      long long bytes;
+      double time;
       Statistics();
+      void Reset();
       void Add(const Statistics &);
+      bool HaveSomethingDone(int);
    };
    Statistics stats;
 
@@ -140,7 +144,8 @@ public:
       IGNORE_TIME=1<<10,
       REMOVE_FIRST=1<<11,
       IGNORE_SIZE=1<<12,
-      NO_SYMLINKS=1<<13
+      NO_SYMLINKS=1<<13,
+      LOOP=1<<14
    };
 
    void SetFlags(int f,int v)
