@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ResMgr.cc,v 1.47.2.1 2003/12/17 09:52:53 lav Exp $ */
+/* $Id: ResMgr.cc,v 1.55 2005/06/21 12:14:05 lav Exp $ */
 
 #include <config.h>
 
@@ -32,9 +32,9 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <errno.h>
-extern "C" {
+CDECL_BEGIN
 #include <regex.h>
-}
+CDECL_END
 #include "ResMgr.h"
 #include "SMTask.h"
 #include "xmalloc.h"
@@ -471,6 +471,11 @@ const char *ResMgr::UNumberValidate(char **value)
    char *v=*value;
 
    v+=strspn(v," \t");
+   memmove(*value,v,strlen(v)+1);   // drop leading space
+
+   v=*value;
+   if(!strncasecmp(v,"0x",2))
+      v+=2;
 
    int n=strspn(v,"1234567890");
 
@@ -642,7 +647,7 @@ ResDecl::~ResDecl()
 #define HOUR   (60*MINUTE)
 #define DAY    (24*HOUR)
 
-TimeInterval::TimeInterval(const char *s)
+void TimeInterval::init(const char *s)
 {
    double interval=0;
    infty=false;
@@ -724,7 +729,7 @@ bool Range::Match(int n)
    return full || (n>=start && n<=end);
 }
 
-#ifndef HAVE_RANDOM_DECL
+#if !HAVE_DECL_RANDOM
 CDECL long random();
 CDECL void srandom(unsigned seed);
 #endif
@@ -770,7 +775,7 @@ const char *ResMgr::ERegExpValidate(char **s)
    return 0;
 }
 
-#if defined(HAVE_INET_ATON) && !defined(HAVE_INET_ATON_DECL)
+#if HAVE_INET_ATON && !HAVE_DECL_INET_ATON
 CDECL int inet_aton(const char *,struct in_addr *);
 #endif
 
@@ -796,34 +801,7 @@ const char *ResMgr::IPv6AddrValidate(char **value)
 }
 #endif
 
-const char *ResMgr::FileReadable(char **value)
-{
-   if(!**value)
-      return 0;
-   const char *f=expand_home_relative(*value);
-   char *cwd=0;
-   const char *error=0;
-   if(f[0]!='/')
-   {
-      cwd=xgetcwd();
-      if(cwd)
-	 f=dir_file(cwd,f);
-   }
-   if(access(f,R_OK)<0)
-      error=strerror(errno);
-   else
-   {
-      if(f!=*value)
-      {
-	 xfree(*value);
-	 *value=xstrdup(f);
-      }
-   }
-   xfree(cwd);
-   return error;
-}
-
-const char *ResMgr::DirReadable(char **value)
+const char *ResMgr::FileAccessible(char **value,int mode,int want_dir)
 {
    if(!**value)
       return 0;
@@ -839,9 +817,9 @@ const char *ResMgr::DirReadable(char **value)
    struct stat st;
    if(stat(f,&st)<0)
       error=strerror(errno);
-   else if(!S_ISDIR(st.st_mode))
-      error=strerror(ENOTDIR);
-   else if(access(f,R_OK|X_OK)<0)
+   else if(want_dir ^ S_ISDIR(st.st_mode))
+      error=strerror(want_dir?ENOTDIR:EISDIR);
+   else if(access(f,mode)<0)
       error=strerror(errno);
    else
    {
@@ -852,6 +830,32 @@ const char *ResMgr::DirReadable(char **value)
       }
    }
    xfree(cwd);
+   return error;
+}
+const char *ResMgr::FileReadable(char **value)
+{
+   return FileAccessible(value,R_OK);
+}
+const char *ResMgr::FileExecutable(char **value)
+{
+   return FileAccessible(value,X_OK);
+}
+const char *ResMgr::DirReadable(char **value)
+{
+   return FileAccessible(value,R_OK|X_OK,1);
+}
+
+CDECL_BEGIN
+#include <iconv.h>
+CDECL_END
+const char *ResMgr::CharsetValidate(char **value)
+{
+   if(!**value)
+      return 0;
+   iconv_t ic=iconv_open(*value,*value);
+   if(ic==(iconv_t)-1)
+      return _("this encoding is not supported");
+   iconv_close(ic);
    return 0;
 }
 
