@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ResMgr.h,v 1.30 2002/05/05 14:23:19 lav Exp $ */
+/* $Id: ResMgr.h,v 1.39 2005/06/21 12:14:06 lav Exp $ */
 
 #ifndef RESMGR_H
 #define RESMGR_H
@@ -99,6 +99,15 @@ public:
    static const char *RangeValidate(char **value);
    static const char *ERegExpValidate(char **value);
    static const char *IPv4AddrValidate(char **value);
+#if INET6
+   static const char *IPv6AddrValidate(char **value);
+#endif
+   static const char *UNumberPairValidate(char **value);
+   static const char *FileAccessible(char **value,int mode,int want_dir=0);
+   static const char *FileReadable(char **value);
+   static const char *FileExecutable(char **value);
+   static const char *DirReadable(char **value);
+   static const char *CharsetValidate(char **value);
    static const char *NoClosure(char **);
    static bool str2bool(const char *value);
 
@@ -141,11 +150,11 @@ public:
       }
    operator int()
       {
-	 return atoi(s);
+	 return strtol(s,0,0);
       }
    operator long()
       {
-	 return atol(s);
+	 return strtol(s,0,0);
       }
    operator double()
       {
@@ -160,6 +169,14 @@ public:
 	 return s;
       }
    bool is_nil() { return s==0; }
+   void ToNumberPair(int &a,int &b)
+      {
+	 switch(sscanf(s,"%d%*c%d",&a,&b))
+	 {
+	 case 0: a=0;
+	 case 1: b=a;
+	 }
+      }
 };
 
 inline bool ResDecl::QueryBool(const char *closure)
@@ -175,8 +192,10 @@ class TimeInterval : public TimeDiff
 {
    bool infty;
    const char *error_text;
+   void init(const char *);
 public:
-   TimeInterval(const char *);
+   TimeInterval(const char *s) { init(s); }
+   TimeInterval(ResValue r) { init(r); }
    TimeInterval(time_t i) : TimeDiff(i,0) { infty=false; error_text=0; }
    TimeInterval(const TimeDiff &d) : TimeDiff(d) { infty=false; error_text=0; }
    bool Error() const { return error_text!=0; };
