@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Resolver.cc,v 1.62 2004/09/06 08:16:51 lav Exp $ */
+/* $Id: Resolver.cc,v 1.64 2005/06/14 13:35:32 lav Exp $ */
 
 #include <config.h>
 
@@ -385,6 +385,11 @@ void Resolver::ParseOrder(const char *s,int *o)
    if(o) o[idx]=-1;
 }
 
+#if defined(HAVE_DN_EXPAND) && !defined(HAVE_DECL_DN_EXPAND)
+CDECL int dn_expand(const unsigned char *msg,const unsigned char *eomorig,const unsigned char *comp_dn,char *exp_dn,int length);
+CDECL int dn_skipname(const unsigned char *msg,const unsigned char *eomorig);
+#endif
+
 #ifdef HAVE_RES_SEARCH
 static
 int extract_domain(const unsigned char *answer,const unsigned char *scan,int len,
@@ -972,7 +977,7 @@ void ResolverCache::CacheCheck()
    while(*scan)
    {
       Entry *s=*scan;
-      TimeInterval expire((const char *)ResMgr::Query("dns:cache-expire",s->hostname));
+      TimeInterval expire(ResMgr::Query("dns:cache-expire",s->hostname));
       if((!expire.IsInfty() && SMTask::now>=s->timestamp+expire.Seconds())
       || (count>=countlimit))
       {
