@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SFtp.cc,v 1.45 2005/05/20 10:14:32 lav Exp $ */
+/* $Id: SFtp.cc,v 1.50 2005/08/04 10:10:29 lav Exp $ */
 
 #include <config.h>
 #include "SFtp.h"
@@ -52,20 +52,12 @@ bool SFtp::GetBetterConnection(int level,bool limit_reached)
       {
 	 if(level<2)
 	    continue;
-	 if(!connection_takeover || (o->priority>=priority && !o->suspended))
+	 if(!connection_takeover || (o->priority>=priority && !o->IsSuspended()))
 	    continue;
 	 o->Disconnect();
 	 return need_sleep;
       }
 
-      if(home && !o->home)
-	 o->home=xstrdup(home);
-      else if(!home && o->home)
-	 home=xstrdup(o->home);
-
-      o->ExpandTildeInCWD();
-      ExpandTildeInCWD();
-
       if(level==0 && xstrcmp(real_cwd,o->real_cwd))
 	 continue;
 
@@ -245,7 +237,7 @@ int SFtp::Do()
       m=MOVED;
 
    case CONNECTED:
-      if(home==0 && !RespQueueIsEmpty())
+      if(home.path==0 && !RespQueueIsEmpty())
 	 return m;
 
       if(mode==CLOSED)
@@ -322,12 +314,13 @@ void SFtp::MoveConnectionHere(SFtp *o)
    if(expect_chain_end==&o->expect_chain)
       expect_chain_end=&expect_chain;
    o->expect_chain_end=&o->expect_chain;
-   event_time=o->event_time;
+   BumpEventTime(o->event_time);
    ssh_id=o->ssh_id;
    state=CONNECTED;
    o->Disconnect();
    if(!home)
       set_home(home_auto);
+   ResumeInternal();
 }
 
 void SFtp::Disconnect()
@@ -360,7 +353,6 @@ void SFtp::Init()
    state=DISCONNECTED;
    send_buf=0;
    recv_buf=0;
-   recv_buf_suspended=false;
    pty_send_buf=0;
    pty_recv_buf=0;
    file_buf=0;
@@ -714,9 +706,8 @@ void SFtp::Close()
    // don't need these out-of-order packets anymore
    while(ooo_chain)
       DeleteExpect(&ooo_chain);
-   if(recv_buf && !suspended)
+   if(recv_buf)
       recv_buf->Resume();
-   recv_buf_suspended=false;
 }
 
 int SFtp::HandlePty()
@@ -837,8 +828,7 @@ void SFtp::HandleExpect(Expect *e)
 	 }
 	 if(mode==CHANGE_DIR && RespQueueIsEmpty())
 	 {
-	    xfree(cwd);
-	    cwd=xstrdup(file);
+	    cwd.Set(file);
 	    eof=true;
 	    LsCache::SetDirectory(this,cwd,true);
 	 }
@@ -1287,36 +1277,27 @@ int SFtp::Done()
    return IN_PROGRESS;
 }
 
-void SFtp::Suspend()
+void SFtp::SuspendInternal()
 {
-   if(suspended)
-      return;
    if(recv_buf)
-   {
-      recv_buf_suspended=recv_buf->IsSuspended();
-      recv_buf->Suspend();
-   }
+      recv_buf->SuspendSlave();
    if(send_buf)
-      send_buf->Suspend();
+      send_buf->SuspendSlave();
    if(pty_send_buf)
-      pty_send_buf->Suspend();
+      pty_send_buf->SuspendSlave();
    if(pty_recv_buf)
-      pty_recv_buf->Suspend();
-   super::Suspend();
+      pty_recv_buf->SuspendSlave();
 }
-void SFtp::Resume()
+void SFtp::ResumeInternal()
 {
-   if(!suspended)
-      return;
-   super::Resume();
-   if(recv_buf && !recv_buf_suspended)
-      recv_buf->Resume();
+   if(recv_buf)
+      recv_buf->ResumeSlave();
    if(send_buf)
-      send_buf->Resume();
+      send_buf->ResumeSlave();
    if(pty_send_buf)
-      pty_send_buf->Resume();
+      pty_send_buf->ResumeSlave();
    if(pty_recv_buf)
-      pty_recv_buf->Resume();
+      pty_recv_buf->ResumeSlave();
 }
 
 const char *SFtp::CurrentStatus()
@@ -2018,17 +1999,15 @@ const char *SFtpDirList::Status()
    return "";
 }
 
-void SFtpDirList::Suspend()
+void SFtpDirList::SuspendInternal()
 {
    if(ubuf)
-      ubuf->Suspend();
-   super::Suspend();
+      ubuf->SuspendSlave();
 }
-void SFtpDirList::Resume()
+void SFtpDirList::ResumeInternal()
 {
-   super::Resume();
    if(ubuf)
-      ubuf->Resume();
+      ubuf->ResumeSlave();
 }
 
 
