@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SFtp.h,v 1.16 2004/10/18 05:45:18 lav Exp $ */
+/* $Id: SFtp.h,v 1.18 2005/07/06 06:09:03 lav Exp $ */
 
 #ifndef SFTP_H
 #define SFTP_H
@@ -141,7 +141,6 @@ private:
 
    IOBuffer *send_buf;
    IOBuffer *recv_buf;
-   bool recv_buf_suspended;
    IOBuffer *pty_send_buf;
    IOBuffer *pty_recv_buf;
    DirectedBuffer *send_translate;
@@ -713,13 +712,13 @@ public:
 
    bool NeedSizeDateBeforehand() { return true; }
 
-   void Suspend();
-   void Resume();
+   void SuspendInternal();
+   void ResumeInternal();
 
    void Cleanup();
    void CleanupThis();
 
-   FileSet *GetFileSet() { FileSet *fset=file_set; file_set=0; return fset; }
+   FileSet *GetFileSet() { FileSet *fset=file_set; file_set=0; return fset?fset:new FileSet; }
 };
 
 class SFtpDirList : public DirList
@@ -737,8 +736,8 @@ public:
    const char *Status();
    int Do();
 
-   void Suspend();
-   void Resume();
+   void SuspendInternal();
+   void ResumeInternal();
 };
 
 class SFtpListInfo : public ListInfo
