@@ -29,14 +29,18 @@ class SMTask
    virtual int Do() = 0;
 
    SMTask *next;
-   SMTask *prev_current;   // stack of running tasks
 
    static SMTask *chain;
    static PollVec sched_total;
+   static SMTask **stack;
+   static int stack_ptr;
+   static int stack_size;
+
+   bool	 suspended;
+   bool	 suspended_slave;
 
 protected:
    int	 running;
-   bool	 suspended;
    bool	 deleting;
 
    enum
@@ -48,6 +52,10 @@ protected:
 
    virtual ~SMTask();
 
+   // SuspendInternal and ResumeInternal usually suspend and resume slave tasks
+   virtual void SuspendInternal() {}
+   virtual void ResumeInternal() {}
+
 public:
    PollVec  block;
 
@@ -61,9 +69,15 @@ public:
    static void Schedule();
    static void Block() { sched_total.Block(); }
 
-   virtual void Suspend();
-   virtual void Resume();
-   bool IsSuspended() { return suspended; }
+   void Suspend();
+   void Resume();
+
+   // SuspendSlave and ResumeSlave are used in SuspendInternal/ResumeInternal
+   // to suspend/resume slave tasks
+   void SuspendSlave();
+   void ResumeSlave();
+
+   bool IsSuspended() { return suspended|suspended_slave; }
 
    virtual void Reconfig(const char *name=0) {};
    static void ReconfigAll(const char *name);
@@ -79,18 +93,8 @@ public:
 
    static SMTask *current;
 
-   static void Enter(SMTask *task)
-      {
-	 task->running++;
-	 task->prev_current=current;
-	 current=task;
-      }
-   static void Leave(SMTask *task)
-      {
-	 current=task->prev_current;
-	 task->prev_current=0;
-	 task->running--;
-      }
+   static void Enter(SMTask *task);
+   static void Leave(SMTask *task);
    void Enter() { Enter(this); }
    void Leave() { Leave(this); }
 
