@@ -228,6 +228,12 @@ void Buffer::SetError(const char *e,bool fatal)
    error_text=xstrdup(e);
    error_fatal=fatal;
 }
+void Buffer::SetErrorCached(const char *e)
+{
+   xfree(error_text);
+   error_text=xasprintf(_("%s [cached]"),e);
+   error_fatal=false;
+}
 #if 0 // unused
 void Buffer::SetError2(const char *e1,const char *e2)
 {
@@ -523,18 +529,20 @@ int IOBufferFDStream::Put_LL(const char *buf,int size)
    res=write(fd,buf,size);
    if(res==-1)
    {
-      if(NonFatalError(errno))
+      saved_errno=errno;
+      if(E_RETRY(saved_errno))
       {
 	 Block(fd,POLLOUT);
 	 return 0;
       }
+      if(NonFatalError(saved_errno))
+	 return 0;
       if(errno==EPIPE)
       {
 	 broken=true;
 	 return -1;
       }
-      saved_errno=errno;
-      stream->MakeErrorText();
+      stream->MakeErrorText(saved_errno);
       goto stream_err;
    }
    if(put_ll_timer)
@@ -564,13 +572,15 @@ int IOBufferFDStream::Get_LL(int size)
    res=read(fd,buffer+buffer_ptr+in_buffer,size);
    if(res==-1)
    {
-      if(NonFatalError(errno))
+      saved_errno=errno;
+      if(E_RETRY(saved_errno))
       {
 	 Block(fd,POLLIN);
 	 return 0;
       }
-      saved_errno=errno;
-      stream->MakeErrorText();
+      if(NonFatalError(saved_errno))
+	 return 0;
+      stream->MakeErrorText(saved_errno);
       goto stream_err;
    }
 
@@ -626,15 +636,13 @@ int IOBufferFileAccess::Get_LL(int size)
    return res;
 }
 
-void IOBufferFileAccess::Suspend()
+void IOBufferFileAccess::SuspendInternal()
 {
-   session->Suspend();
-   super::Suspend();
+   session->SuspendSlave();
 }
-void IOBufferFileAccess::Resume()
+void IOBufferFileAccess::ResumeInternal()
 {
-   super::Resume();
-   session->Resume();
+   session->ResumeSlave();
 }
 const char *IOBufferFileAccess::Status()
 {
