@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: buffer.h,v 1.36 2005/04/04 09:59:43 lav Exp $ */
+/* $Id: buffer.h,v 1.37 2005/06/22 08:04:22 lav Exp $ */
 
 #ifndef BUFFER_H
 #define BUFFER_H
@@ -166,7 +166,7 @@ public:
       }
    virtual time_t EventTime()
       {
-	 if(suspended)
+	 if(IsSuspended())
 	    return now;
 	 return event_time;
       }
@@ -230,8 +230,8 @@ protected:
 public:
    IOBufferFileAccess(FileAccess *i) : IOBuffer(GET) { session=i; }
 
-   void Suspend();
-   void Resume();
+   void SuspendInternal();
+   void ResumeInternal();
    const char *Status();
 };
 
