@@ -18,36 +18,28 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: buffer_ssl.h,v 1.3 2003/12/22 18:19:28 lav Exp $ */
+/* $Id: buffer_ssl.h,v 1.5 2005/05/18 05:57:08 lav Exp $ */
 
 #ifndef BUFFER_SSL_H
 #define BUFFER_SSL_H
 
 #include "buffer.h"
 
-#ifdef USE_SSL
-# include <openssl/ssl.h>
+#if USE_SSL
+#include "lftp_ssl.h"
 class IOBufferSSL : public IOBuffer
 {
-   SSL *ssl;
-   bool ssl_connected;
-   bool do_connect;
+   lftp_ssl *ssl;
    bool close_later;
 
-   char *hostname;
-
    int Get_LL(int size);
    int Put_LL(const char *buf,int size);
 
-   bool IsFatal(int res);
-
 protected:
    ~IOBufferSSL();
 
 public:
-   IOBufferSSL(SSL *s,dir_t m,const char *h=0);
-   void DoConnect()	{ ssl_connected=false; do_connect=true; }
-   void SetConnected()	{ ssl_connected=true; }
+   IOBufferSSL(lftp_ssl *s,dir_t m);
    void CloseLater()	{ close_later=true; }
    int Do();
 };
