@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: history.cc,v 1.10.2.1 2003/12/17 09:52:55 lav Exp $ */
+/* $Id: history.cc,v 1.12 2005/08/03 07:12:48 lav Exp $ */
 
 #include <config.h>
 #include <sys/types.h>
@@ -140,13 +140,20 @@ void History::Close()
    }
 }
 
-void History::Set(FileAccess *s,const char *cwd)
+void History::Set(FileAccess *s,const FileAccess::Path &cwd)
 {
-   if(cwd==0 || !strcmp(cwd,"~") || s->GetHostName()==0)
+   if(cwd.path==0 || !strcmp(cwd.path,"~") || s->GetHostName()==0)
       return;
-   char *res=(char*)alloca(32+strlen(cwd)*3+1);
+   char *res=(char*)alloca(32+strlen(cwd.path)*3+xstrlen(cwd.url)+2);
    sprintf(res,"%lu:",(unsigned long)time(0));
-   url::encode_string(cwd,res+strlen(res),URL_PATH_UNSAFE);
+   if(!cwd.url)
+   {
+      url::encode_string(cwd,res+strlen(res),URL_PATH_UNSAFE);
+      if(!cwd.is_file && url::dir_needs_trailing_slash(s->GetProto()))
+	 strcat(res,"/");
+   }
+   else
+      strcat(res,cwd.url);
    super::Add(s->GetConnectURL(s->NO_PATH|s->NO_PASSWORD),res);
    modified=true;
 }
