@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: misc.cc,v 1.72 2005/04/28 22:59:39 lav Exp $ */
+/* $Id: misc.cc,v 1.73 2005/06/21 12:17:21 lav Exp $ */
 
 #include <config.h>
 #include "xmalloc.h"
@@ -266,7 +266,7 @@ void  truncate_file_tree(const char *dir)
       SignalHook::Ignore(SIGTSTP);
       SignalHook::Ignore(SIGQUIT);
       SignalHook::Ignore(SIGHUP);
-      execlp("rm","rm","-rf",dir,NULL);
+      execlp("rm","rm","-rf",dir,(char*)NULL);
       perror("execlp(rm)");
       fflush(stderr);
       _exit(1);
