@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: resource.cc,v 1.116 2005/05/18 05:57:10 lav Exp $ */
+/* $Id: resource.cc,v 1.120 2005/07/04 13:55:40 lav Exp $ */
 
 #include <config.h>
 
@@ -34,6 +34,7 @@
 #include "url.h"
 #include "GetPass.h"
 #include "ascii_ctype.h"
+#include "confpaths.h"
 
 static const char *FtpProxyValidate(char **p)
 {
@@ -186,6 +187,7 @@ static ResDecl
    ResDecl00 ("ftp:anon-pass",		  "lftp@", 0,0),
    ResDecl01 ("ftp:anon-user",		  "anonymous",0,0),
    ResDecl01a("ftp:auto-sync-mode",	  "",	   ResMgr::ERegExpValidate,0),
+   ResDecl01i("ftp:auto-passive-mode",	  "yes",   ResMgr::BoolValidate,0),
    ResDecl01b("ftp:bind-data-socket",	  "yes",   ResMgr::BoolValidate,0),
    ResDecl01f("ftp:charset",		  "",	   ResMgr::CharsetValidate,0),
    ResDecl01g("ftp:client",		  PACKAGE"/"VERSION,0,0),
@@ -290,7 +292,7 @@ static ResDecl
 #if INET6
    ResDecl42b("net:socket-bind-ipv6",	  "",	   ResMgr::IPv6AddrValidate,0),
 #endif
-   ResDecl43 ("net:timeout",		  "300",   ResMgr::UNumberValidate,0),
+   ResDecl43 ("net:timeout",		  "5m",   ResMgr::TimeIntervalValidate,0),
    ResDecl44 ("net:connection-limit",	  "0",	   ResMgr::UNumberValidate,0),
    ResDecl45 ("net:connection-takeover",  "yes",   ResMgr::BoolValidate,0),
    ResDecl46 ("mirror:order",		  "*.sfv *.sig *.md5* *.sum * */", 0,ResMgr::NoClosure),
@@ -326,7 +328,7 @@ static ResDecl
 #endif
 
 #if INET6
-# define DEFAULT_ORDER "inet inet6"
+# define DEFAULT_ORDER "inet6 inet"
 #else
 # define DEFAULT_ORDER "inet"
 #endif
@@ -347,6 +349,10 @@ static ResDecl
 static ResDecl
    res_dir_colors ("color:dir-colors",   "",   0,ResMgr::NoClosure);
 
+static ResDecl
+   res_verify	  ("xfer:verify", "no", ResMgr::BoolValidate,ResMgr::NoClosure),
+   res_verify_cmd ("xfer:verify-command","",ResMgr::FileExecutable,0);
+
 #ifdef HAVE_LANGINFO_H
 # include <langinfo.h>
 #endif
@@ -420,4 +426,6 @@ void ResMgr::ClassInit()
    if(cs)
       Set("file:charset",0,cs);
 #endif
+
+   Set("xfer:verify-command",0,PKGDATADIR"/verify-file");
 }
